package lawOfDemeter;

import java.util.*;

/**
 * The Any class is part of a Law of Demeter Checker - a set of
 * classes and aspects that verify a given program does not violate
 * the Law of Demeter.  This class contains all of the pointcuts that
 * need to be examined in an application to verify it meets the law of demeter.
 *
 * @author David H. Lorenz
 * @author Modifications made by Paul Freeman
 */
abstract class Any {
  // we encapsulate all pointcuts we need in a class and
  // refer to them later as Any.*

  // not in this package and either
  //  - not within the controlflow of a call in this package
  // OR
  //  - within the control flow of a method call in the Tests aspect of this package
  private pointcut scope(): !within(lawOfDemeter..*) &&
    (!cflow(execution(* lawOfDemeter..*(..))) ||
     cflow(execution(* lawOfDemeter.Tests.*(..))));

  // capturing method calls.
  pointcut ConstructorCall(): scope() && call(*.new(..));
  pointcut MethodCall(): scope() && call(* *(..));

  // capturing method executions
  pointcut MainMethodExecution(): scope() && execution(void *.main(String[]));
  pointcut MethodExecution(): scope() && execution(* *(..));
  pointcut ConstructorExecution(): scope() && execution(*.new (..));
  pointcut Execution(): ConstructorExecution() || MethodExecution();

  // capturing assignments to data members
  pointcut Set(): scope() && set(* *.*) ;

  // capturing accesses of data members
  pointcut Get(): scope() && get(* *.*);
}
